import datetime

from faker import Faker
import pandas as pd
import numpy as np

np.random.seed(4321)

Faker.seed(4321)

fake = Faker()

## Create 1000 records
num_registros=1000

df = pd.DataFrame()
#df = df[df['ccaa_cd'].isin(["01","02"])]

# patient_id

df["patient_id"] = [fake.unique.pystr(min_chars=6, max_chars=6) for i in range(num_registros)]


# hospital_cd and hospital_st
# National Hospital Catalogue

df1 = pd.read_csv('./inputs/cat_cnh_202303301704.csv', sep='|')
#df["hospital_cd"] = range(x)
#df["hospital_st"] = range(x)

s = df1[["codcnh","nombre_centro"]].sample(n=num_registros,replace=True)


s = s.rename(columns={"codcnh":"hospital_cd","nombre_centro":"hospital_st"})

df.reset_index(drop=True, inplace=True)
s.reset_index(drop=True, inplace=True)

df = pd.concat([df,s],axis=1)

# ccaa_cd
# NUTS 2021 codes

df["ccaa_cd"] = "ES24"


# healthcare_area_cd

df["healthcare_area_cd"] = [fake.numerify(text='####') for i in range(num_registros)]

# age_nm
# age_nm >= 18 | age_nm <= 115

df["age_nm"] = [fake.random_int(min=18, max=115) for i in range(num_registros)]

# sex_cd
#  0 "not known", 1 "male", 2 "female", 9 "not applicable"

# df["sex_cd"] = [fake.random_sample(elements=(0,1,2,9), length=1) for i in range(num_registros)]

sex_cd = pd.DataFrame([0,1,2,9], columns=['sex_cd'])
s = sex_cd.sample(n=num_registros,replace=True)

df.reset_index(drop=True, inplace=True)
s.reset_index(drop=True, inplace=True)

df = pd.concat([df,s],axis=1)


# municipality_code_cd and zip_code_cd


df1 = pd.read_csv('./inputs/cat_cod_postales_202303311216.csv', sep='|')
#df["hospital_cd"] = range(x)
#df["hospital_st"] = range(x)

s = df1[["codigo_postal","municipio_id"]].sample(n=num_registros,replace=True)


s = s.rename(columns={"codigo_postal":"zip_code_cd","municipio_id":"municipality_code_cd"})

df.reset_index(drop=True, inplace=True)
s.reset_index(drop=True, inplace=True)

df = pd.concat([df,s],axis=1)

# type_admission_cd
# 1 "Unplanned", 2 "Planned"

df["type_admission_cd"] = [fake.random_int(min=1, max=2) for i in range(num_registros)]



#### Generate exitus_dt in order to be able to set the dates

df["exitus_dt"] = [fake.date_time_between_dates(datetime_start=datetime.datetime(year=2010, month=1, day=1), datetime_end=datetime.datetime(year=2021, month=12, day=31)) for i in range(num_registros)]

#### Hospital dates

# hospital_admission_date_dt

df["hospital_admission_date_dt"] = df["exitus_dt"].apply(lambda x: fake.date_time_between_dates(datetime_start=datetime.datetime(year=2010, month=1, day=1), datetime_end=x))

# hospital_intervention_date_dt

df["hospital_intervention_date_dt"] = df["hospital_admission_date_dt"].apply(lambda x:fake.date_time_between_dates(datetime_start=x,
                                                                                                     datetime_end=x + datetime.timedelta(days=30)))

df.loc[df["exitus_dt"] <= df["hospital_intervention_date_dt"], "hospital_intervention_date_dt"] = df["exitus_dt"]

# hospital_discharge_date_dt

df["hospital_discharge_date_dt"] = df["hospital_intervention_date_dt"].apply(lambda x:fake.date_time_between_dates(datetime_start=x,
                                                                                                     datetime_end=x + datetime.timedelta(days=30)))
df.loc[df["exitus_dt"] <= df["hospital_discharge_date_dt"], "hospital_discharge_date_dt"] = df["exitus_dt"]

# hospital_type_discharge_cd

hospital_type_discharge_cd = pd.DataFrame([1,2,3,4,5,8,9], columns=['hospital_type_discharge_cd'])
s = hospital_type_discharge_cd.sample(n=num_registros,replace=True)

df.reset_index(drop=True, inplace=True)
s.reset_index(drop=True, inplace=True)

df = pd.concat([df,s],axis=1)

# hospital_discharge_type_cd = 4 (exitus), hospital_discharge_dt = exitus_dt

df.loc[df["hospital_type_discharge_cd"] == 4, "exitus_dt"] = df["hospital_discharge_date_dt"]


# inhospital_trombectomy_bl

df["inhospital_trombectomy_bl"] = 1

# inhospital_fibrinolysis_bl

df["inhospital_fibrinolysis_bl"] = 1

# type_ischemic_stroke_bl

#  0 "arterial stenosis or occlusion", 1 "thrombus/embolism"

df["type_ischemic_stroke_bl"] = [fake.random_int(min=0, max=1) for i in range(num_registros)]

# socioeconomic_level_cd
# 1 "TSI001: Free pharmacy", 2 "TSI002-01: 10% limit copayment (pensioners type I)", 3 "TSI002-02: 10% limit copaymen (pensioners type II)",
# 4 "TSI003: 40% limit copayment (insured type I, income <18,000)", 5  "TSI004: 50% limit copayment (insured type II, income between 18,000-100,000)",
# 6 "TSI005: 60% limit copayment (income >=100,000)", 7 "TSI005-03: 60% limit copayment (pensioners with highest income)",
# 8 "TSI006: 30% limit copayment (Mutualist - MUFACE, MUGEJU, ISFAS)"

df["socioeconomic_level_cd"] = [fake.random_int(min=1, max=8) for i in range(num_registros)]


#### Emergency dates


# emergency_care_exitus_dt

# df["emergency_care_exitus_dt"] = df["exitus_dt"]

# admission_emergency_care_dt

df["admission_emergency_care_dt"] = df["exitus_dt"].apply(lambda x: fake.date_time_between_dates(datetime_start=datetime.datetime(year=2010, month=1, day=1, hour= 0, minute = 0, second = 0), datetime_end=x))

# triage_emergency_care_dt

df["triage_emergency_care_dt"] = df["admission_emergency_care_dt"].apply(lambda x:fake.date_time_between_dates(datetime_start=x,
                                                                                                     datetime_end=x + datetime.timedelta(days=1)))

df.loc[df["exitus_dt"] <= df["triage_emergency_care_dt"], "triage_emergency_care_dt"] = df["exitus_dt"]

# first_asisstance_medical_dt

df["first_asisstance_medical_dt"] = df["triage_emergency_care_dt"].apply(lambda x:fake.date_time_between_dates(datetime_start=x,
                                                                                                     datetime_end=x + datetime.timedelta(days=1)))

df.loc[df["exitus_dt"] <= df["first_asisstance_medical_dt"], "first_asisstance_medical_dt"] = df["exitus_dt"]

# admission_to_observation_ward_dt

df["admission_to_observation_ward_dt"] = df["first_asisstance_medical_dt"].apply(lambda x:fake.date_time_between_dates(datetime_start=x,
                                                                                                     datetime_end=x + datetime.timedelta(days=1)))

df.loc[df["exitus_dt"] <= df["admission_to_observation_ward_dt"], "admission_to_observation_ward_dt"] = df["exitus_dt"]


# internal_neurology_consultation_dt

df["internal_neurology_consultation_dt"] = df["first_asisstance_medical_dt"].apply(lambda x:fake.date_time_between_dates(datetime_start=x,
                                                                                                     datetime_end=x + datetime.timedelta(days=1)))

df.loc[df["exitus_dt"] <= df["internal_neurology_consultation_dt"], "internal_neurology_consultation_dt"] = df["exitus_dt"]


# ct_mri_dt

df["ct_mri_dt"] = df["first_asisstance_medical_dt"].apply(lambda x:fake.date_time_between_dates(datetime_start=x,
                                                                                                     datetime_end=x + datetime.timedelta(days=1)))

df.loc[df["exitus_dt"] <= df["ct_mri_dt"], "ct_mri_dt"] = df["exitus_dt"]

# ct_inhospital_bl

df["ct_inhospital_bl"] = 1

# mri_inhospital_bl

df["mri_inhospital_bl"] = 1

# thrombolysis_emergency_dt

df["thrombolysis_emergency_dt"] = df["first_asisstance_medical_dt"].apply(lambda x:fake.date_time_between_dates(datetime_start=x,
                                                                                                     datetime_end=x + datetime.timedelta(days=1)))

df.loc[df["exitus_dt"] <= df["thrombolysis_emergency_dt"], "thrombolysis_emergency_dt"] = df["exitus_dt"]


# discharge_from_emergency_dt

df["discharge_from_emergency_dt"] = df["thrombolysis_emergency_dt"].apply(lambda x:fake.date_time_between_dates(datetime_start=x,
                                                                                                     datetime_end=x + datetime.timedelta(days=3)))

df.loc[df["exitus_dt"] <= df["discharge_from_emergency_dt"], "discharge_from_emergency_dt"] = df["exitus_dt"]

# type_discharge_emergency_bl

df["type_discharge_emergency_bl"] = 1

# n_admission_prior_emergency_nm máximo???

df["n_admission_prior_emergency_nm"] = [fake.random_int(min=1, max=10000) for i in range(num_registros)]

# n_admission_prior_inhospital_nm máximo???

df["n_admission_prior_inhospital_nm"] = [fake.random_int(min=1, max=10000) for i in range(num_registros)]

# date_first_readmissions_30days_all_cause_dt

df["date_first_readmissions_30days_all_cause_dt"] = df["hospital_discharge_date_dt"].apply(lambda x:fake.date_time_between_dates(datetime_start=x,
                                                                                                     datetime_end=x + datetime.timedelta(days=30)))

df.loc[df["exitus_dt"] <= df["date_first_readmissions_30days_all_cause_dt"], "date_first_readmissions_30days_all_cause_dt"] = df["exitus_dt"]

# n_readmissions_30days_all_cause_bl

df["n_readmissions_30days_all_cause_bl"] = 1



# anticoagulants_prescriptions_bl

df["anticoagulants_prescriptions_bl"] = 1

# atc_code_anticoagulants_cd

df1 = pd.read_csv('./inputs/cat_anticoagulants_prescriptions_bl.csv', sep='|')
#df["hospital_cd"] = range(x)
#df["hospital_st"] = range(x)

s = df1["clean_code"].sample(n=num_registros,replace=True)


df.reset_index(drop=True, inplace=True)
s.reset_index(drop=True, inplace=True)

df["atc_code_anticoagulants_cd"] = s





# start_date_prescription_anticoagulants_dt

df["start_date_prescription_anticoagulants_dt"] = df["exitus_dt"].apply(lambda x:fake.date_time_between_dates(datetime_start=datetime.datetime(year=2010, month=1, day=1),
                                                                                                     datetime_end=x))

# end_date_prescription_anticoagulants_dt

df["end_date_prescription_anticoagulants_dt"] = df[["start_date_prescription_anticoagulants_dt","exitus_dt"]].apply(lambda x:fake.date_time_between_dates(
    datetime_start=x["start_date_prescription_anticoagulants_dt"], datetime_end=x["exitus_dt"]), axis=1)


# antiarrhythmics_prescription_bl

df["antiarrhythmics_prescription_bl"] = 1

# atc_code_antiarrhythmic_cd


df1 = pd.read_csv('./inputs/cat_antiarrhythmics_prescription_bl.csv', sep='|')
#df["hospital_cd"] = range(x)
#df["hospital_st"] = range(x)

s = df1["clean_code"].sample(n=num_registros,replace=True)


df.reset_index(drop=True, inplace=True)
s.reset_index(drop=True, inplace=True)

df["atc_code_antiarrhythmic_cd"] = s




# start_date_prescription_antiarrhythmic_dt

df["start_date_prescription_antiarrhythmic_dt"] = df["exitus_dt"].apply(lambda x:fake.date_time_between_dates(datetime_start=datetime.datetime(year=2010, month=1, day=1),
                                                                                                     datetime_end=x))

# end_date_prescription_antiarrhythmic_dt

df["end_date_prescription_antiarrhythmic_dt"] = df[["start_date_prescription_antiarrhythmic_dt","exitus_dt"]].apply(lambda x:fake.date_time_between_dates(
    datetime_start=x["start_date_prescription_antiarrhythmic_dt"], datetime_end=x["exitus_dt"]), axis=1)


# antihypertensive_prescription_bl

df["antihypertensive_prescription_bl"] = 1

# atc_code_antihypertensive_cd

df1 = pd.read_csv('./inputs/cat_antihypertensive_prescription_bl.csv', sep='|')
#df["hospital_cd"] = range(x)
#df["hospital_st"] = range(x)

s = df1["clean_code"].sample(n=num_registros,replace=True)


df.reset_index(drop=True, inplace=True)
s.reset_index(drop=True, inplace=True)

df["atc_code_antihypertensive_cd"] = s


# start_date_prescription_antihypertensive_dt

df["start_date_prescription_antihypertensive_dt"] = df["exitus_dt"].apply(lambda x:fake.date_time_between_dates(datetime_start=datetime.datetime(year=2010, month=1, day=1),
                                                                                                     datetime_end=x))

# end_date_prescription_antihypertensive_dt

df["end_date_prescription_antihypertensive_dt"] = df[["start_date_prescription_antihypertensive_dt","exitus_dt"]].apply(lambda x:fake.date_time_between_dates(
    datetime_start=x["start_date_prescription_antihypertensive_dt"], datetime_end=x["exitus_dt"]), axis=1)

# antiaggregants_prescription_bl

df["antiaggregants_prescription_bl"] = 1

# atc_code_antiaggregants_cd

df1 = pd.read_csv('./inputs/cat_antiaggregants_prescription_bl.csv', sep='|')
#df["hospital_cd"] = range(x)
#df["hospital_st"] = range(x)

s = df1["clean_code"].sample(n=num_registros,replace=True)


df.reset_index(drop=True, inplace=True)
s.reset_index(drop=True, inplace=True)

df["atc_code_antiaggregants_cd"] = s




# start_date_prescription_antiaggregants_dt

df["start_date_prescription_antiaggregants_dt"] = df["exitus_dt"].apply(lambda x:fake.date_time_between_dates(datetime_start=datetime.datetime(year=2010, month=1, day=1),
                                                                                                     datetime_end=x))

# end_date_prescription_antiaggregants_dt

df["end_date_prescription_antiaggregants_dt"] = df[["start_date_prescription_antiaggregants_dt","exitus_dt"]].apply(lambda x:fake.date_time_between_dates(
    datetime_start=x["start_date_prescription_antiaggregants_dt"], datetime_end=x["exitus_dt"]), axis=1)


# fibrinolitics_prescriptions_bl

df["fibrinolitics_prescriptions_bl"] = 1

# falta atc_code_fibrinolitics_cd

df1 = pd.read_csv('./inputs/cat_fibrinolitics_prescriptions_bl.csv', sep='|')
#df["hospital_cd"] = range(x)
#df["hospital_st"] = range(x)

s = df1["clean_code"].sample(n=num_registros,replace=True)


df.reset_index(drop=True, inplace=True)
s.reset_index(drop=True, inplace=True)

df["atc_code_fibrinolitics_cd"] = s




# start_date_prescription_fibrinolitics_dt

#df["start_date_prescription_fibrinolitics_dt"] = df["exitus_dt"].apply(lambda x:fake.date_time_between_dates(datetime_start=datetime.datetime(year=2010, month=1, day=1),
#                                                                                                     datetime_end=x))

# end_date_prescription_fibrinolitics_dt

#df["end_date_prescription_fibrinolitics_dt"] = df[["start_date_prescription_fibrinolitics_dt","exitus_dt"]].apply(lambda x:fake.date_time_between_dates(
#   datetime_start=x["start_date_prescription_fibrinolitics_dt"], datetime_end=x["exitus_dt"]), axis=1)


# nihss_score_nm

df["nihss_score_nm"] = [fake.random_int(min=1, max=5) for i in range(num_registros)]



# nihss_score_date_dt

df["nihss_score_date_dt"] = df["admission_emergency_care_dt"]


# modified_rankin_scale_cd

df["modified_rankin_scale_cd"] = [fake.random_int(min=0, max=8) for i in range(num_registros)]



# modified_rankin_scale_dt

df["modified_rankin_scale_dt"] = df["hospital_intervention_date_dt"]


# barthel_index_nm

# Options: 1-< 20 Total; 2-20-35 Severe; 3-40-55 Moderate; 4-≥ 60 Mild; 5-100 Independent

df["barthel_index_nm"] = [fake.random_int(min=1, max=5) for i in range(num_registros)]



# barthel_index_dt

df["barthel_index_dt"] = df["discharge_from_emergency_dt"]

# exitus_bl

df["exitus_bl"] = 1

# heart_failure_bl

df["heart_failure_bl"] = [fake.random_int(min=0, max=1) for i in range(num_registros)]

# hypertension_bl

df["hypertension_bl"] = [fake.random_int(min=0, max=1) for i in range(num_registros)]

# diabetes_bl

df["diabetes_bl"] = [fake.random_int(min=0, max=1) for i in range(num_registros)]


# atrial_fibrillation_bl

df["atrial_fibrillation_bl"] = [fake.random_int(min=0, max=1) for i in range(num_registros)]

# valvular_disease_bl

df["valvular_disease_bl"] = [fake.random_int(min=0, max=1) for i in range(num_registros)]

# hospital_icu_admission_bl

df["hospital_icu_admission_bl"] = 1

# hospital_icu_admission_dt

# df["hospital_icu_admission_dt"] = df["exitus_dt"].apply(lambda x: fake.date_time_between_dates(datetime_start=datetime.datetime(year=2010, month=1, day=1), datetime_end=x))

# hospital_icu_stay

df["hospital_icu_stay"] = [fake.random_int(min=0, max=15) for i in range(num_registros)]

df = df[["patient_id",                         "hospital_cd",                                 "hospital_st","ccaa_cd",
"healthcare_area_cd",                          "age_nm",                                      "sex_cd",                                     "municipality_code_cd",
"zip_code_cd",                                 "type_admission_cd",                           "hospital_admission_date_dt",                  "hospital_intervention_date_dt",
"hospital_discharge_date_dt",                  "hospital_type_discharge_cd",                  "inhospital_trombectomy_bl",                   "inhospital_fibrinolysis_bl",
"type_ischemic_stroke_bl",                     "socioeconomic_level_cd",                      "admission_emergency_care_dt",                 "triage_emergency_care_dt",
"first_asisstance_medical_dt",                 "admission_to_observation_ward_dt",            "internal_neurology_consultation_dt",          "ct_mri_dt",
"ct_inhospital_bl",                            "mri_inhospital_bl",                           "thrombolysis_emergency_dt",                   "discharge_from_emergency_dt",
"type_discharge_emergency_bl",                 "exitus_dt",                                   "n_admission_prior_emergency_nm",              "n_admission_prior_inhospital_nm",
"date_first_readmissions_30days_all_cause_dt", "n_readmissions_30days_all_cause_bl",          "anticoagulants_prescriptions_bl",             "atc_code_anticoagulants_cd",
"start_date_prescription_anticoagulants_dt",   "end_date_prescription_anticoagulants_dt",     "antiarrhythmics_prescription_bl",             "atc_code_antiarrhythmic_cd",
"start_date_prescription_antiarrhythmic_dt",   "end_date_prescription_antiarrhythmic_dt",    "antihypertensive_prescription_bl",            "atc_code_antihypertensive_cd",
"start_date_prescription_antihypertensive_dt", "end_date_prescription_antihypertensive_dt",   "antiaggregants_prescription_bl",             "atc_code_antiaggregants_cd",
"start_date_prescription_antiaggregants_dt",   "end_date_prescription_antiaggregants_dt",     "fibrinolitics_prescriptions_bl",              "atc_code_fibrinolitics_cd",
"nihss_score_nm",                              "nihss_score_date_dt",                         "modified_rankin_scale_cd",                    "modified_rankin_scale_dt",
"barthel_index_nm",                            "barthel_index_dt",                            "exitus_bl",                                   "heart_failure_bl",
"hypertension_bl",                             "diabetes_bl",                                 "atrial_fibrillation_bl",                      "valvular_disease_bl",
"hospital_icu_admission_bl",                   "hospital_icu_stay"]]


df.to_csv("./outputs/synthetic_data_concept_stroke.csv", sep='|', index=False)